self.__precacheManifest = (self.__precacheManifest || []).concat([
  {
    "revision": "74cdf85775ecedc288792f8178816fbd",
    "url": "/index.html"
  },
  {
    "revision": "88af1e82dbe511aab0fa",
    "url": "/static/css/main.1fc973a9.chunk.css"
  },
  {
    "revision": "0b999f215fd073ff61d8",
    "url": "/static/js/2.4064d2ac.chunk.js"
  },
  {
    "revision": "89b0379e7bcda1a468d8b0343aeb4e53",
    "url": "/static/js/2.4064d2ac.chunk.js.LICENSE.txt"
  },
  {
    "revision": "88af1e82dbe511aab0fa",
    "url": "/static/js/main.a065b716.chunk.js"
  },
  {
    "revision": "98c1327435369df6cc45",
    "url": "/static/js/runtime-main.c63fbe4a.js"
  },
  {
    "revision": "cabc4b0d6882524fc0e7872c7fc50e0d",
    "url": "/static/media/logo.cabc4b0d.png"
  }
]);